<?php

declare(strict_types=1);

namespace Dedi\SyliusDpdPickUpPlugin\Service;

use Dedi\SyliusDpdPickUpPlugin\Entity\OrderInterface;
use Dedi\SyliusDpdPickUpPlugin\Entity\ShippingMethodInterface;
use Dedi\SyliusDpdPickUpPlugin\Exception\MissingPudoException;
use Dedi\SyliusDpdPickUpPlugin\Model\Pudo;
use Sylius\Component\Core\Model\Address;
use Sylius\Component\Core\Model\ShipmentInterface;

class PudoSelector
{
    /**
     * Update order to set the selected pudo if any.
     *
     * @throw MissingPudoException
     */
    public function __invoke(
        OrderInterface $order,
        ?Pudo $pudo
    ): void {
        /** @var ShipmentInterface $shippment */
        $shippment = $order->getShipments()->first();

        /** @var ShippingMethodInterface $method */
        $method = $shippment->getMethod();

        if (!$pudo && $method->isDpdPickUp()) {
            throw new MissingPudoException();
        }

        if (!$pudo || !$method->isDpdPickUp()) {
            $order->setPudoId(null);

            return;
        }

        /** @var Address $address */
        $address = $order->getShippingAddress();
        $address->setCity($pudo->city);
        $address->setPostcode($pudo->city);
        $address->setPostcode($pudo->zipCode);

        $street = $pudo->address1;
        if ($pudo->address2) {
            $street .= sprintf(', %s', $pudo->address2);
        }
        if ($pudo->address3) {
            $street .= sprintf(', %s', $pudo->address3);
        }
        $address->setStreet($street);

        $order->setPudoId($pudo->id);
    }
}
